@@ -175,6 +175,8 @@ class HTTPClient(object):
 
 class MetaCatClient(HTTPClient, TokenAuthClientMixin):
     
+    Version = "1.0"
+    
     def __init__(self, server_url=None, auth_server_url=None, max_concurrent_queries = 5,
                 token = None, token_file = None, timeout = None):    
 
@@ -206,15 +208,15 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
             if isinstance(token, (str, bytes)):
                 token = SignedToken.decode(token)
             self.Token = token
-            
-        if token is None:
-            self.TokenLib = TokenLib()
-            token = self.TokenLib.get(server_url)
-        
+
         server_url = server_url or os.environ.get("METACAT_SERVER_URL")
         if not server_url:
             raise RuntimeError("MetaCat server URL unspecified")
-        
+
+        if token is None:
+            self.TokenLib = TokenLib()
+            token = self.TokenLib.get(server_url)
+
         HTTPClient.__init__(self, server_url, token, timeout)
         self.AuthURL = auth_server_url or server_url + "/auth"
         self.MaxConcurrent = max_concurrent_queries
@@ -361,19 +363,25 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         Parameters
         ----------
         dataset : str
-            "namespace:name"
+            "namespace:name" or "name", if namespace argument is given
         file_list : list
             List of dictionaries, one dictionary per file. Each dictionary must contain either a file id
         
             .. code-block:: python
         
-                    { "fid": "..." }
+                    { "fid": "abcd12345" }
 
-            or a file namespace/name:
+            or namespace/name:
         
             .. code-block:: python
 
-                    { "name": "namespace:name" }
+                    { "name": "filename.data", "namespace": "my_namespace" }
+
+            or DID:
+        
+            .. code-block:: python
+
+                    { "did": "my_namespace:filename.data" }
         
         namespace : str, optional
             Default namespace. If a ``file_list`` item is specified with a name without a namespace, the ``default namespace``
@@ -384,10 +392,33 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
         list
             list of dictionaries, one dictionary per file with file ids: { "fid": "..." }
         """        
+            
+        default_namespace = namespace
+        if ':' not in dataset:
+            if default_namespace is None:
+                raise ValueError("Namespace not specified for the target dataset")
+            dataset = f"{default_namespace}:{dataset}"
+
         url = f"data/add_files?dataset={dataset}"
-        if namespace:
-            url += f"&namespace={namespace}"
-        out = self.post_json(url, file_list)
+        
+        data = []
+        for f in file_list:
+            if "fid" in f:
+                f = {"fid":f["fid"]}
+            elif "did" in f:
+                namespace, name = parse_name(f["did"], default_namespace)
+                if namespace is None:
+                    raise ValueError("Namespace not specified for file with did=" + f["did"])
+                f = {"namespace":namespace, "name":name}
+            elif "name" in f:
+                namespace = f.get("namespace") or default_namespace
+                if not namespace:
+                    raise ValueError("Namespace not specified for file with name=" + f["name"])
+                f = {"namespace":namespace, "name":f["name"]}
+            else:
+                raise ValueError("Infalid file specification: %s. Must contain either did or namespace/name or fid" % (f,))
+            data.append(f)
+        out = self.post_json(url, data)
         return out
 
     def declare_files(self, dataset, files, namespace=None):
@@ -425,11 +456,11 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
                     "name": "filename",                 # optional,
                     "did": "namespace:filename",        # optional, convenience for Rucio users
                                                         # either "did" or "name", "namespace" must be present
-                    "fid":  "...",                      # optional, file id. Will be auto-generated if unspecified.
-                                                        # if specified, must be unique for the instance
-                    "parents": ["fid","fid",...],       # optional, list of parent file ids
+                    "size": ...,                        # required, integer number of bytes, must be > 0
                     "metadata": {...},                  # optional, file metadata, a dictionary with arbitrary JSON'able contents
-                    "size": ...,                        # optional, integer number of bytes
+                    "fid":  "...",                      # optional, file id. Will be auto-generated if unspecified.
+                                                        # if specified, must be unique
+                    "parents": ["fid","fid",...],       # optional, list of parent file ids
                     "checksums": {                      # optional, checksums dictionary
                         "method": "value",...
                     }
@@ -444,30 +475,27 @@ class MetaCatClient(HTTPClient, TokenAuthClientMixin):
 
         for i, item in enumerate(files):
             f = item.copy()
+            namespace = f.get("namespace", default_namespace)
             if "did" in f:
-                did = f.pop("did")
                 if "name" in f or "namespace" in f:
                     raise ValueError(f"Both DID and namespace/name specified for {did}")
+                did = f.pop("did")
                 namespace, name = parse_name(did, default_namespace)
-            elif "name" in f:
-                name = f["name"]
-                namespace = f.get("namespace", default_namespace)
-                if not namespace:
-                    raise ValueError(f"Namespace not specified for {name}")
-            else:
-                raise ValueError(f"A file without a name or DID found at index {i}")
+                f["name"] = name
             f["namespace"] = namespace
-            f["name"] = name
+            size = f.get("size")
+            if not isinstance(size, int) or size < 0:
+                raise ValueError("File size is unspecified or invalid for for {namespace}:{name} (#{i} in the list)")
 
-            for k in f.get("metadata", {}).keys():
+            meta = item.get("metadata", {})
+            for k in meta.keys():
                 if '.' not in k:
-                    raise ValueError(f'Invalid metadata key "{k}" for {namespace}:{name} (#{i}): must contain dot (.)')
+                    raise ValueError(f'Invalid metadata key "{k}" for file #{i} in the list: metadata key must contain dot (.)')
 
+            f["metadata"] = meta
             lst.append(f)
 
         url = f"data/declare_files?dataset={dataset}"
-        if namespace:
-            url += f"&namespace={namespace}"
         out = self.post_json(url, lst)
         return out
 
